(in-package :asn.1)

(defvar *mib-nodes* nil)

(defun children-function (x)
  (let ((children (list-children x)))
    (sort (delete nil children)
          #'(lambda (a b) (< (oid-value a) (oid-value b))))))

(defun print-child (x)
  (when x
    (let ((name (oid-name x))
          (id (oid-value x)))
      (format nil "~A (~D)" name id))))

(defun display-graph-selection (interface data)
  (with-slots (display-pane-name) interface
    (setf (capi:display-pane-text display-pane-name)
          (format nil "~A (~D)" (oid-name data) (oid-value data)))))

(defun search-callback (interface data)
  (let ((g (mibrowser-mib-graph interface))
        (current-node data))
    (when current-node
      (setf (capi:choice-selected-item g) current-node)
      (display-graph-selection interface current-node))))

(defclass mibrowser-node (capi:expandable-item-pinboard-object)
  ()
  (:default-initargs))

(capi:define-interface mibrowser ()
  ()
  (:panes
   (mib-graph capi:graph-pane
              :node-pinboard-class 'mibrowser-node
              :children-function 'children-function
              :print-function 'print-child
              :reader mibrowser-mib-graph
              :roots (list-children *root-object-id*)
              :callback-type :interface-data
              :selection-callback 'display-graph-selection
              :visible-min-height 300)
   (search capi:text-input-pane
           :title "Search:"
           :text "iso"
           :callback 'search-callback
           :buttons (list :ok t)
           :callback-type :interface-data
           :visible-min-width '(:character 10))
   (title-pane-name capi:title-pane :text "Name (OID):")
   (display-pane-name capi:display-pane))
  (:layouts
   (mibrowser-layout capi:column-layout '(head-layout mib-graph))
   (head-layout capi:row-layout '(search title-pane-name display-pane-name)
                :y-adjust :center))
  (:menu-bar file)
  (:menus (file "File" ("Quit")))
  (:default-initargs
   :layout 'mibrowser-layout
   :best-width 800
   :best-height 600
   :title "MIB Browser"))

(defmethod capi:interface-display :before ((self mibrowser))
  (let ((g (mibrowser-mib-graph self))
        (start-node |iso|))
    (setf (capi:choice-selected-item g) start-node
          *mib-nodes* (capi:graph-pane-nodes g))
    (display-graph-selection self start-node)))

(defun mibrowser ()
  (capi:display (make-instance 'mibrowser)))
