(in-package :asn.1)

(deftype access ()
  '(member :not-accessible
           :accessible-for-notify
           :read-only
           :read-write
           :read-create))

(deftype status ()
  '(member :current :deprecated :obsolete))

(deftype type-specifier ()
  '(or symbol list))

(defclass object-id (asn.1-type)
  ((name        :type string
                :reader oid-name
                :initarg :name
                :initform "")
   (value       :type integer
                :reader oid-value
                :initarg :value
                :initform 0)
   (syntax      :type type-specifier
                :reader oid-type
                :initarg :syntax)
   (max-access  :type access
                :reader oid-max-access
                :initarg :max-access)
   (status      :type status
                :reader oid-status
                :initarg :status)
   (description :type string
                :reader oid-description
                :initarg :description)
   (module      :type symbol
                :reader oid-module
                :initarg :module)
   (parent      :type object-id
                :reader oid-parent
                :initarg :parent)
   (children    :type hash-table
                :accessor oid-children
                :initform (make-hash-table)))
  (:documentation "OBJECT IDENTIFIER"))

(defmethod print-object ((object object-id) stream)
  (print-unreadable-object (object stream :type t)
    (format stream "~A (~D) [~D]"
            (oid-name object)
            (oid-value object)
            (hash-table-count (oid-children object)))))

(defmethod initialize-instance :after ((instance object-id)
                                       &rest initargs &key &allow-other-keys)
  (if (slot-boundp instance 'parent)
      (let ((o (oid-parent instance)))
        (setf (gethash (oid-value instance) (oid-children o)) instance))
    (warn "Didn't have a parent?!")))

(defun list-children (oid)
  (declare (type object-id oid))
  (let ((all-children '()))
    (maphash #'(lambda (key value)
                 (declare (ignore key))
                 (push value all-children))
             (oid-children oid))
    all-children))

(defmacro delete-object (oid)
  `(progn
     (setf (gethash (oid-value ,oid)
                    (oid-children (oid-parent ,oid)))
           nil)
     (makunbound ',oid)))

(defvar *root-object-id* (make-instance 'object-id :value 0))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (defmacro defoid (name (parent value))
    `(progn
       (defvar ,name (make-instance 'object-id
                                    :name (symbol-name ',name)
                                    :value ,value
                                    :parent ,parent))
       (export ',name))))

(defoid |iso| (*root-object-id* 1))
